<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:xs="http://www.w3.org/2001/XMLSchema"
                xmlns:a371="http://www.infrabel.be/Artemis/Infrastructure/20080711"
                xmlns:shared="urn:shared-functions"
                xmlns:local="urn:local-functions"
                version="2.0">
   <xsl:import href="shared-functions.xslt"/>
   <xsl:param name="region-file" as="xs:string"/>
   <xsl:param name="purge-date" as="xs:string"/>
   <xsl:output method="xml" version="1.0" encoding="UTF-8" indent="yes"/>
   <xsl:strip-space elements="*"/>
   <xsl:variable name="region-xml" select="doc($region-file)"/>
   <xsl:key name="by-id" match="*" use="@id"/>
   <!-- *********************************************************************************************************** -->
   <xsl:template match="a371:ptcars">
      <xsl:element name="a371:ptcars">
         <xsl:sequence select="@*"/>
         <xsl:apply-templates/>
      </xsl:element>
   </xsl:template>
   <!-- *********************************************************************************************************** -->
   <xsl:template match="a371:ptcar">
      <xsl:if test="@validToDate ge $purge-date">
         <xsl:variable name="splitPtcar" select="local:completeAndSplitPtcar(.)"/>
         <xsl:sequence select="local:transformToExportPtcar($splitPtcar)"/>
      </xsl:if>
   </xsl:template>
   <!-- *********************************************************************************************************** -->
   <xsl:function name="local:completeAndSplitPtcar" as="element()*">
      <xsl:param name="ptcar" as="element()"/>
      <xsl:for-each select="$ptcar">
         <xsl:variable name="region" select="local:getRegion(.)"/>
         <xsl:sequence select="local:splitPtcar(., $region)"/>
      </xsl:for-each>
   </xsl:function>
   <!-- *********************************************************************************************************** -->
   <xsl:function name="local:getRegion" as="element()*">
      <xsl:param name="ptcar" as="element()"/>
      <xsl:for-each select="$ptcar">
         <xsl:sequence select="local:getRegion(@additionalRegionIdc, @validFromDate, @validToDate)"/>
      </xsl:for-each>
   </xsl:function>
   <!-- *********************************************************************************************************** -->
   <xsl:function name="local:getRegion" as="element()*">
      <xsl:param name="id" as="xs:integer?"/>
      <xsl:param name="validFromDate" as="xs:string"/>
      <xsl:param name="validToDate" as="xs:string"/>
      <xsl:if test="exists($id)">
         <xsl:sequence select="key('by-id', string($id), $region-xml)[$validToDate ge @validFromDate and @validToDate ge $validFromDate and @validToDate ge $purge-date]"/>
      </xsl:if>
   </xsl:function>
   <!-- *********************************************************************************************************** -->
   <xsl:function name="local:splitPtcar" as="element()*">
      <xsl:param name="pPtcar" as="element()"/>
      <xsl:param name="pRegion" as="element()*"/>
      <xsl:variable name="validities" select="shared:get-validities($pPtcar, $pRegion, $purge-date)"/>
      <xsl:for-each select="$validities">
         <xsl:variable name="validFromDate" select="@validFromDate"/>
         <xsl:variable name="validToDate" select="@validToDate"/>
         <xsl:variable name="region" select="$pRegion[$validToDate ge @validFromDate and @validToDate ge $validFromDate]"/>
         <xsl:for-each select="$pPtcar">
            <xsl:element name="a371:ptcar">
               <xsl:sequence select="@* except (@validFromDate, @validToDate)"/>
               <xsl:attribute name="validFromDate" select="$validFromDate"/>
               <xsl:attribute name="validToDate" select="$validToDate"/>
               <xsl:sequence select="$region"/>
            </xsl:element>
         </xsl:for-each>
      </xsl:for-each>
   </xsl:function>
   <!-- *********************************************************************************************************** -->
   <xsl:function name="local:transformToExportPtcar" as="element()*">
      <xsl:param name="ptcar" as="element()*"/>
      <xsl:for-each select="$ptcar">
         <xsl:copy>
            <xsl:sequence select="@*"/>
            <xsl:for-each select="a371:region">
               <xsl:attribute name="area" select="@shortNameDutch"/>
            </xsl:for-each>
         </xsl:copy>
      </xsl:for-each>
   </xsl:function>
   <!-- *********************************************************************************************************** -->
</xsl:stylesheet>
