<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:xs="http://www.w3.org/2001/XMLSchema"
                xmlns:output="http://www.w3.org/2010/xslt-xquery-serialization"
                xmlns:saxon="http://saxon.sf.net/"
                xmlns:file="http://expath.org/ns/file"
                xmlns:map="http://www.w3.org/2005/xpath-functions/map"
                xmlns:shared="urn:shared-functions"
                extension-element-prefixes="saxon"
                exclude-result-prefixes="#all"
                version="2.0">
   <xsl:output method="xml" version="1.0" encoding="UTF-8" indent="yes"/>
   <xsl:variable name="TAB" select="'&#x9;'"/>
   <xsl:variable name="dummy-doc" select="shared:dummy-doc()"/>
   <xsl:variable name="output-params" select="shared:output-params()"/>
   <!-- *********************************************************************************************************** -->
   <xsl:function name="shared:dummy-doc" as="document-node()">
      <xsl:document>
         <dummy/>
      </xsl:document>
   </xsl:function>
   <!-- *********************************************************************************************************** -->
   <xsl:function name="shared:extract-validities" as="element()*">
      <xsl:param name="entityVersions" as="element()*"/>
      <xsl:param name="pRefValidFromDate" as="xs:string"/>
      <xsl:param name="pRefValidToDate" as="xs:string"/>
      <xsl:variable name="validFromDates" as="xs:string*"
                    select="distinct-values($entityVersions/@validFromDate[. gt $pRefValidFromDate and . lt $pRefValidToDate])"/>
      <xsl:variable name="validToDates" as="xs:string*"
                    select="distinct-values($entityVersions/@validToDate[. gt $pRefValidFromDate and . lt $pRefValidToDate])"/>
      <xsl:variable name="shiftedValidFromDates" as="xs:string*" select="for $date in $validFromDates return (shared:previous-day($date))"/>
      <xsl:variable name="shiftedValidToDates" as="xs:string*" select="for $date in $validToDates return (shared:next-day($date))"/>
      <xsl:variable name="mergedValidFromDates" as="xs:string*" select="distinct-values(($pRefValidFromDate, $validFromDates, $shiftedValidToDates))"/>
      <xsl:variable name="mergedValidToDates" as="xs:string*" select="distinct-values(($validToDates, $shiftedValidFromDates, $pRefValidToDate))"/>
      <xsl:variable name="dates" as="xs:string*">
         <xsl:for-each select="$mergedValidFromDates, $mergedValidToDates">
            <xsl:sort select="."/>
            <xsl:sequence select="."/>
         </xsl:for-each>
      </xsl:variable>
      <xsl:for-each select="$dates">
         <xsl:variable name="index" select="position()"/>
         <xsl:if test="($index mod 2) = 1">
            <xsl:element name="shared:validity">
               <xsl:attribute name="validFromDate" select="."/>
               <xsl:attribute name="validToDate" select="$dates[$index + 1]"/>
            </xsl:element>
         </xsl:if>
      </xsl:for-each>
   </xsl:function>
   <!-- *********************************************************************************************************** -->
   <xsl:function name="shared:get-validities" as="element()*">
      <xsl:param name="element" as="element()"/>
      <xsl:param name="entityVersions" as="element()*"/>
      <xsl:for-each select="$element">
         <xsl:sequence select="shared:extract-validities($entityVersions, @validFromDate, @validToDate)"/>
      </xsl:for-each>
   </xsl:function>
   <!-- *********************************************************************************************************** -->
   <xsl:function name="shared:get-validities" as="element()*">
      <xsl:param name="element" as="element()"/>
      <xsl:param name="entityVersions" as="element()*"/>
      <xsl:param name="purgeDate" as="xs:string"/>
      <xsl:sequence select="shared:get-validities($element, $entityVersions)[@validToDate ge $purgeDate]"/>
   </xsl:function>
   <!-- *********************************************************************************************************** -->
   <xsl:function name="shared:is-absolute-uri" as="xs:boolean">
      <xsl:param name="uri" as="xs:string?"/>
      <xsl:sequence select="matches($uri,'^[a-z]+:')"/>
   </xsl:function>
   <!-- *********************************************************************************************************** -->
   <xsl:function name="shared:log-transform-options">
      <xsl:param name="options" as="map(*)"/>
      <xsl:variable name="source-location" select="map:get($options, 'source-location')"/>
      <xsl:variable name="base-output-uri" select="map:get($options, 'base-output-uri')"/>
      <xsl:variable name="stylesheet-location" select="map:get($options, 'stylesheet-location')"/>
      <xsl:variable name="stylesheet-params" select="map:get($options, 'stylesheet-params')"/>
      <xsl:choose>
         <xsl:when test="exists($source-location) and exists($base-output-uri)">
            <xsl:message select="concat('Transforming ',$source-location)"/>
            <xsl:message select="concat($TAB,'into ',$base-output-uri)"/>
            <xsl:message select="concat($TAB,'using ',$stylesheet-location)"/>
         </xsl:when>
         <xsl:when test="exists($source-location) and empty($base-output-uri)">
            <xsl:message select="concat('Transforming ',$source-location)"/>
            <xsl:message select="concat($TAB,'using ',$stylesheet-location)"/>
         </xsl:when>
         <xsl:when test="empty($source-location) and exists($base-output-uri)">
            <xsl:message select="concat('Generating ',$base-output-uri)"/>
            <xsl:message select="concat($TAB,'using ',$stylesheet-location)"/>
         </xsl:when>
         <xsl:otherwise>
            <xsl:message select="concat('Launching ',$stylesheet-location)"/>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:if test="exists($stylesheet-params)">
         <xsl:for-each select="map:keys($stylesheet-params)">
            <xsl:sort select="string(.)"/>
            <xsl:variable name="value" select="map:get($stylesheet-params, .)"/>
            <xsl:variable name="parameter-value" select="if ($value instance of document-node()) then document-uri($value) else string-join($value, ' ')"/>
            <xsl:message select="concat($TAB,'parameter ',.,'=',$parameter-value)"/>
         </xsl:for-each>
      </xsl:if>
   </xsl:function>
   <!-- *********************************************************************************************************** -->
   <xsl:function name="shared:next-day" as="xs:string">
      <xsl:param name="date"/>
      <xsl:sequence select="shared:shift-day($date, 1)"/>
   </xsl:function>
   <!-- *********************************************************************************************************** -->
   <xsl:function name="shared:output-params" as="element()">
      <output:serialization-parameters>
         <output:method value="xml"/>
         <output:indent value="yes"/>
      </output:serialization-parameters>
   </xsl:function>
   <!-- *********************************************************************************************************** -->
   <xsl:function name="shared:previous-day" as="xs:string">
      <xsl:param name="date"/>
      <xsl:sequence select="shared:shift-day($date, -1)"/>
   </xsl:function>
   <!-- *********************************************************************************************************** -->
   <xsl:function name="shared:shift-day" as="xs:string">
      <xsl:param name="date"/>
      <xsl:param name="nbOfDays"/>
      <xsl:variable name="pDate" select="xs:date($date)"/>
      <xsl:variable name="pNbOfDays" select="xs:integer($nbOfDays)"/>
      <xsl:choose>
         <xsl:when test="$pNbOfDays gt 0">
            <!-- Add abs to avoid compilation warning -->
            <xsl:value-of select="$pDate + xs:dayTimeDuration(concat('P',abs($pNbOfDays),'D'))"/>
         </xsl:when>
         <xsl:when test="$pNbOfDays lt 0">
            <xsl:value-of select="$pDate - xs:dayTimeDuration(concat('P',abs($pNbOfDays),'D'))"/>
         </xsl:when>
         <xsl:otherwise>
            <xsl:value-of select="$pDate"/>
         </xsl:otherwise>
      </xsl:choose>
   </xsl:function>
   <!-- *********************************************************************************************************** -->
   <xsl:template name="shared:transform">
      <xsl:param name="options" as="map(*)"/>
      <xsl:call-template name="shared:write">
         <xsl:with-param name="output" select="shared:transform($options)"/>
      </xsl:call-template>
   </xsl:template>
   <!-- *********************************************************************************************************** -->
   <xsl:function name="shared:transform" as="map(*)">
      <xsl:param name="options" as="map(*)"/>
      <xsl:sequence select="shared:log-transform-options($options)"/>
      <xsl:sequence select="transform($options)"/>
   </xsl:function>
   <!-- *********************************************************************************************************** -->
   <xsl:template name="shared:transform-with-saxon">
      <xsl:param name="options" as="map(*)"/>
      <xsl:call-template name="shared:write">
         <xsl:with-param name="output" select="shared:transform-with-saxon($options)"/>
      </xsl:call-template>
   </xsl:template>
   <!-- *********************************************************************************************************** -->
   <xsl:function name="shared:transform-with-saxon" as="map(*)">
      <xsl:param name="options" as="map(*)"/>
      <xsl:sequence select="shared:log-transform-options($options)"/>
      <xsl:variable name="stylesheet" select="saxon:compile-stylesheet(doc($options?stylesheet-location))"/>
      <xsl:variable name="source" select="if (exists($options?source-location)) then doc($options?source-location) else $dummy-doc"/>
      <xsl:variable name="base-output-uri" select="($options?base-output-uri, 'output')[1]"/>
      <xsl:map-entry key="$base-output-uri" select="saxon:transform($stylesheet, $source, $options?stylesheet-params)"/>
   </xsl:function>
   <!-- *********************************************************************************************************** -->
   <xsl:template name="shared:write">
      <xsl:param name="output" as="map(*)"/>
      <!-- Exclude default added "output" when base-output-uri is absent by checking is-absolute-uri() -->
      <xsl:for-each select="map:keys($output)[shared:is-absolute-uri(.)]">
         <xsl:sort select="."/>
         <xsl:message select="concat('Exporting ', .)"/>
         <xsl:sequence select="file:write(., map:get($output, .), $output-params)"/>
      </xsl:for-each>
   </xsl:template>
   <!-- *********************************************************************************************************** -->
</xsl:stylesheet>